#include "proxy.h"

#define maxDgramSize 4096

Proxy::Proxy(QObject* parent): QObject(parent), dgramBuffer(*new QByteArray(maxDgramSize, 0)) {
    socket = new QUdpSocket(this);
    socket->bind(QHostAddress::AnyIPv4, 22000);

    addClient("192.168.0.3", 12015);

    addClient("192.168.0.4", 12015);
    addClient("192.168.0.2", 2001);
    addClient("192.168.0.2", 2002);
    addClient("192.168.0.2", 2003);
    addClient("192.168.0.2", 2004);
    addClient("192.168.0.2", 2005);
    addClient("192.168.0.2", 2006);
    addClient("192.168.0.5", 5001);
    addClient("192.168.0.5", 5002);
    addClient("192.168.0.5", 5003);
    addClient("192.168.0.5", 5504);
    addClient("192.168.0.5", 5005);

    connect(socket, &QUdpSocket::readyRead, this, &Proxy::readPendingDatagrams);
}

void Proxy::readPendingDatagrams() {
    while(socket->hasPendingDatagrams()) {
        qint64 readSize = socket->readDatagram(dgramBuffer.data(), maxDgramSize);

        sendToClients(dgramBuffer.left(readSize));
    }
}

void Proxy::addClient(const QString& addr, quint16 port) {
    // thread per sender model
    // does not make any difference if all senders will reside in a single dedicated thread

    auto* thread = new QThread(this);
    auto* relay = new Relay(addr, port);

    relay->moveToThread(thread);
    connect(this, &Proxy::sendToClients, relay, &Relay::sendToClient);
    thread->start();
}
