#ifndef PROXY_H
#define PROXY_H

#include <QByteArray>
#include <QDebug>
#include <QHostAddress>
#include <QObject>
#include <QThread>
#include <QUdpSocket>

class Relay: public QObject {
    Q_OBJECT
public:
    Relay(const QString& addr, quint16 port, QObject* parent = nullptr):
        QObject(parent),
        addr(QHostAddress(addr)),
        port(port) {
        s = new QUdpSocket(this);
    }

public slots:
    void sendToClient(const QByteArray& data) {
        qint64 writeSize = s->writeDatagram(data.data(), addr, port);

        if(writeSize < data.size() || writeSize == -1) {
            qDebug() << "bad things happen";
        }

        // kind of resolves the issue on current work load
        // QThread::usleep(100);
    }

private:
    QUdpSocket* s;
    QHostAddress addr;
    quint16 port;
};

class Proxy: public QObject {
    Q_OBJECT

public:
    Proxy(QObject* parent = nullptr);

signals:
    void setup();
    void sendToClients(const QByteArray& data);

public slots:
    void readPendingDatagrams();

private:
    QUdpSocket* socket;
    QByteArray& dgramBuffer;

    void addClient(const QString& addr, quint16 port);
};

#endif // PROXY_H
